"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


PostAtsJobsJobIDApplicationsPositiveResponseOutcome = Literal[
    "PENDING",
    "HIRED",
    "DECLINED",
]
r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
Kombo will always try to deliver this information as reliably as possible.
"""


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer7TypedDict(TypedDict):
    raw: NotRequired[Nullable[Any]]
    r"""We pass the original question data along so you can handle it."""


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer7(BaseModel):
    raw: OptionalNullable[Any] = UNSET
    r"""We pass the original question data along so you can handle it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["raw"])
        nullable_fields = set(["raw"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionUnknownTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["UNKNOWN"]
    r"""When we're not able to map a specific question type yet, we will return this type. Every `UNKNOWN` question will also be parsed and unified by us at some point."""


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionUnknown(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["UNKNOWN"], AfterValidator(validate_const("UNKNOWN"))],
        pydantic.Field(alias="type"),
    ] = "UNKNOWN"
    r"""When we're not able to map a specific question type yet, we will return this type. Every `UNKNOWN` question will also be parsed and unified by us at some point."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer7TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer7TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionUnknownTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer7(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer7

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionUnknown


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer6TypedDict(TypedDict):
    date_: Nullable[datetime]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer6(BaseModel):
    date_: Annotated[Nullable[datetime], pydantic.Field(alias="date")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionDateTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["DATE"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionDate(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["DATE"], AfterValidator(validate_const("DATE"))],
        pydantic.Field(alias="type"),
    ] = "DATE"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer6TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer6TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionDateTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer6(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer6

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionDate


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer5TypedDict(TypedDict):
    number: Nullable[float]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer5(BaseModel):
    number: Nullable[float]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionNumberTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["NUMBER"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionNumber(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["NUMBER"], AfterValidator(validate_const("NUMBER"))],
        pydantic.Field(alias="type"),
    ] = "NUMBER"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer5TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer5TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionNumberTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer5(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer5

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionNumber


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer4TypedDict(TypedDict):
    checked: Nullable[bool]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer4(BaseModel):
    checked: Nullable[bool]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionBooleanTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["BOOLEAN"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionBoolean(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["BOOLEAN"], AfterValidator(validate_const("BOOLEAN"))],
        pydantic.Field(alias="type"),
    ] = "BOOLEAN"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer4TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer4TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionBooleanTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer4(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer4

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionBoolean


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer3TypedDict(TypedDict):
    choices: NotRequired[List[str]]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer3(BaseModel):
    choices: Optional[List[str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["choices"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionMultiSelectTypedDict(
    TypedDict
):
    remote_id: Nullable[str]
    title: str
    type: Literal["MULTI_SELECT"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionMultiSelect(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[
            Literal["MULTI_SELECT"], AfterValidator(validate_const("MULTI_SELECT"))
        ],
        pydantic.Field(alias="type"),
    ] = "MULTI_SELECT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer3TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer3TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionMultiSelectTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer3(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer3

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionMultiSelect


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer2TypedDict(TypedDict):
    choice: Nullable[str]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer2(BaseModel):
    choice: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionSingleSelectTypedDict(
    TypedDict
):
    remote_id: Nullable[str]
    title: str
    type: Literal["SINGLE_SELECT"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionSingleSelect(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[
            Literal["SINGLE_SELECT"], AfterValidator(validate_const("SINGLE_SELECT"))
        ],
        pydantic.Field(alias="type"),
    ] = "SINGLE_SELECT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer2TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer2TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionSingleSelectTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer2(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer2

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionSingleSelect


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer1TypedDict(TypedDict):
    content: Nullable[str]


class PostAtsJobsJobIDApplicationsPositiveResponseAnswer1(BaseModel):
    content: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionTextTypedDict(TypedDict):
    remote_id: Nullable[str]
    title: str
    type: Literal["TEXT"]


class PostAtsJobsJobIDApplicationsPositiveResponseQuestionText(BaseModel):
    remote_id: Nullable[str]

    title: str

    TYPE: Annotated[
        Annotated[Literal["TEXT"], AfterValidator(validate_const("TEXT"))],
        pydantic.Field(alias="type"),
    ] = "TEXT"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer1TypedDict(
    TypedDict
):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer1TypedDict
    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionTextTypedDict


class PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer1(BaseModel):
    answer: PostAtsJobsJobIDApplicationsPositiveResponseAnswer1

    question: PostAtsJobsJobIDApplicationsPositiveResponseQuestionText


PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict = TypeAliasType(
    "PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict",
    Union[
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer1TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer2TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer3TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer4TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer5TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer6TypedDict,
        PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer7TypedDict,
    ],
)


PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnion = (
    TypeAliasType(
        "PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnion",
        Union[
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer1,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer2,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer3,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer4,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer5,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer6,
            PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswer7,
        ],
    )
)


class PostAtsJobsJobIDApplicationsPositiveResponseCurrentStageTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    name: Nullable[str]
    r"""The application stage name. For example, \"Initial Screening\"."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    index: Nullable[int]


class PostAtsJobsJobIDApplicationsPositiveResponseCurrentStage(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    name: Nullable[str]
    r"""The application stage name. For example, \"Initial Screening\"."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    index: Nullable[int]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseJobTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    name: Nullable[str]
    r"""Title of the job."""
    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""


class PostAtsJobsJobIDApplicationsPositiveResponseJob(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    name: Nullable[str]
    r"""Title of the job."""

    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponsePhoneNumberTypedDict(TypedDict):
    phone_number: str
    type: NotRequired[Nullable[str]]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""


class PostAtsJobsJobIDApplicationsPositiveResponsePhoneNumber(BaseModel):
    phone_number: str

    type: OptionalNullable[str] = UNSET
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["type"])
        nullable_fields = set(["type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseEmailAddressTypedDict(TypedDict):
    type: Nullable[str]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""
    email_address: NotRequired[Nullable[str]]


class PostAtsJobsJobIDApplicationsPositiveResponseEmailAddress(BaseModel):
    type: Nullable[str]
    r"""Kombo exposes type information through this field. If we don't get any information from the tool, we will set this to `null`."""

    email_address: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["email_address"])
        nullable_fields = set(["email_address", "type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseSocialMediaTypedDict(TypedDict):
    link: NotRequired[Nullable[str]]
    type: NotRequired[Nullable[str]]
    username: NotRequired[Nullable[str]]


class PostAtsJobsJobIDApplicationsPositiveResponseSocialMedia(BaseModel):
    link: OptionalNullable[str] = UNSET

    type: OptionalNullable[str] = UNSET

    username: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["link", "type", "username"])
        nullable_fields = set(["link", "type", "username"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseLocationTypedDict(TypedDict):
    r"""Location of the candidate."""

    city: NotRequired[Nullable[str]]
    country: NotRequired[Nullable[str]]
    r"""Contains the ISO2 country code if possible. If not, it contains the original value."""
    raw: NotRequired[Nullable[str]]
    r"""If we have address data, this is filled with the raw address string."""
    state: NotRequired[Nullable[str]]
    street_1: NotRequired[Nullable[str]]
    r"""If we can parse the address data, this field contains the first part of the street information."""
    street_2: NotRequired[Nullable[str]]
    zip_code: NotRequired[Nullable[str]]


class PostAtsJobsJobIDApplicationsPositiveResponseLocation(BaseModel):
    r"""Location of the candidate."""

    city: OptionalNullable[str] = UNSET

    country: OptionalNullable[str] = UNSET
    r"""Contains the ISO2 country code if possible. If not, it contains the original value."""

    raw: OptionalNullable[str] = UNSET
    r"""If we have address data, this is filled with the raw address string."""

    state: OptionalNullable[str] = UNSET

    street_1: OptionalNullable[str] = UNSET
    r"""If we can parse the address data, this field contains the first part of the street information."""

    street_2: OptionalNullable[str] = UNSET

    zip_code: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["city", "country", "raw", "state", "street_1", "street_2", "zip_code"]
        )
        nullable_fields = set(
            ["city", "country", "raw", "state", "street_1", "street_2", "zip_code"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseTagTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    name: Nullable[str]
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""


class PostAtsJobsJobIDApplicationsPositiveResponseTag(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    name: Nullable[str]

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseCandidateTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    first_name: Nullable[str]
    r"""First name of the candidate."""
    last_name: Nullable[str]
    r"""Last name of the candidate."""
    company: Nullable[str]
    r"""The current company of the candidate."""
    title: Nullable[str]
    r"""The current job title of the candidate."""
    confidential: Nullable[bool]
    r"""Whether the candidate’s profile is confidential in the ATS."""
    source: Nullable[str]
    r"""The hiring source of the candidate. If you're a job board or recruiting service, you can use this to validate which candidates applied through your service and ensure that the correct referral compensation is paid out."""
    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""
    remote_url: Nullable[str]
    r"""URL to the candidate profile in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated candidate profile URL is available, so you might need to fall back to the application URL."""
    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    tags: List[PostAtsJobsJobIDApplicationsPositiveResponseTagTypedDict]
    phone_numbers: NotRequired[
        Nullable[List[PostAtsJobsJobIDApplicationsPositiveResponsePhoneNumberTypedDict]]
    ]
    r"""A list of phone numbers of the candidate."""
    email_addresses: NotRequired[
        Nullable[
            List[PostAtsJobsJobIDApplicationsPositiveResponseEmailAddressTypedDict]
        ]
    ]
    r"""A list of email addresses of the candidate with an optional type. If an email address is invalid, it will be filtered out."""
    social_media: NotRequired[
        Nullable[List[PostAtsJobsJobIDApplicationsPositiveResponseSocialMediaTypedDict]]
    ]
    r"""List of social media accounts of the candidate."""
    location: NotRequired[
        Nullable[PostAtsJobsJobIDApplicationsPositiveResponseLocationTypedDict]
    ]
    r"""Location of the candidate."""


class PostAtsJobsJobIDApplicationsPositiveResponseCandidate(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: str
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    first_name: Nullable[str]
    r"""First name of the candidate."""

    last_name: Nullable[str]
    r"""Last name of the candidate."""

    company: Nullable[str]
    r"""The current company of the candidate."""

    title: Nullable[str]
    r"""The current job title of the candidate."""

    confidential: Nullable[bool]
    r"""Whether the candidate’s profile is confidential in the ATS."""

    source: Nullable[str]
    r"""The hiring source of the candidate. If you're a job board or recruiting service, you can use this to validate which candidates applied through your service and ensure that the correct referral compensation is paid out."""

    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""

    remote_url: Nullable[str]
    r"""URL to the candidate profile in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated candidate profile URL is available, so you might need to fall back to the application URL."""

    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    tags: List[PostAtsJobsJobIDApplicationsPositiveResponseTag]

    phone_numbers: OptionalNullable[
        List[PostAtsJobsJobIDApplicationsPositiveResponsePhoneNumber]
    ] = UNSET
    r"""A list of phone numbers of the candidate."""

    email_addresses: OptionalNullable[
        List[PostAtsJobsJobIDApplicationsPositiveResponseEmailAddress]
    ] = UNSET
    r"""A list of email addresses of the candidate with an optional type. If an email address is invalid, it will be filtered out."""

    social_media: OptionalNullable[
        List[PostAtsJobsJobIDApplicationsPositiveResponseSocialMedia]
    ] = UNSET
    r"""List of social media accounts of the candidate."""

    location: OptionalNullable[PostAtsJobsJobIDApplicationsPositiveResponseLocation] = (
        UNSET
    )
    r"""Location of the candidate."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["phone_numbers", "email_addresses", "social_media", "location"]
        )
        nullable_fields = set(
            [
                "first_name",
                "last_name",
                "company",
                "title",
                "confidential",
                "source",
                "phone_numbers",
                "email_addresses",
                "social_media",
                "location",
                "custom_fields",
                "remote_url",
                "remote_created_at",
                "remote_updated_at",
                "remote_deleted_at",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseDataTypedDict(TypedDict):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""
    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""
    outcome: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseOutcome]
    r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
    Kombo will always try to deliver this information as reliably as possible.
    """
    rejection_reason_name: Nullable[str]
    r"""Reason for the rejection of the candidate."""
    rejected_at: Nullable[datetime]
    r"""The time that the application was rejected.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    current_stage_id: Nullable[str]
    r"""ID of the current application stage"""
    job_id: Nullable[str]
    r"""The Kombo ID of the job which the candidate applied to. The ID can be used to retrieve the job from the `get jobs` endpoint."""
    candidate_id: Nullable[str]
    r"""The Kombo ID of the candidate who applied to the job. The ID can be used to retrieve the candidate from the `get candidates` endpoint."""
    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""
    remote_url: Nullable[str]
    r"""URL to the application in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated application URL is available, so you might need to fall back to the candidate URL."""
    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    current_stage: Nullable[
        PostAtsJobsJobIDApplicationsPositiveResponseCurrentStageTypedDict
    ]
    job: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseJobTypedDict]
    candidate: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseCandidateTypedDict]
    screening_question_answers: NotRequired[
        Nullable[
            List[
                PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnionTypedDict
            ]
        ]
    ]
    r"""A list of answers to screening questions. The screening answer type `FILE` is currently unsupported."""


class PostAtsJobsJobIDApplicationsPositiveResponseData(BaseModel):
    id: str
    r"""The globally unique ID of this object generated by Kombo. We recommend using this as a stable primary key for syncing."""

    remote_id: Nullable[str]
    r"""The raw ID of the object in the remote system. We don't recommend using this as a primary key on your side as it might sometimes be compromised of multiple identifiers if a system doesn't provide a clear primary key."""

    outcome: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseOutcome]
    r"""Parsed status of the application. If Kombo identifies that the application was accepted and the candidate hired, it will be `HIRED`. If the application was rejected or the candidate declined, it will be `DECLINED`. If the application is still in process, it will be `PENDING`.
    Kombo will always try to deliver this information as reliably as possible.
    """

    rejection_reason_name: Nullable[str]
    r"""Reason for the rejection of the candidate."""

    rejected_at: Nullable[datetime]
    r"""The time that the application was rejected.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    current_stage_id: Nullable[str]
    r"""ID of the current application stage"""

    job_id: Nullable[str]
    r"""The Kombo ID of the job which the candidate applied to. The ID can be used to retrieve the job from the `get jobs` endpoint."""

    candidate_id: Nullable[str]
    r"""The Kombo ID of the candidate who applied to the job. The ID can be used to retrieve the candidate from the `get candidates` endpoint."""

    custom_fields: Nullable[Dict[str, Any]]
    r"""A key-value store of fields not covered by the schema. [Read more](/custom-fields)"""

    remote_url: Nullable[str]
    r"""URL to the application in the source ATS system. When embedding this in your system, please consider that not for all ATS a dedicated application URL is available, so you might need to fall back to the candidate URL."""

    changed_at: datetime
    r"""The timestamp when this specific record was last modified. This field only updates when properties directly on this record change, NOT when related or nested models change. For filtering that considers nested data changes, use the `updated_after` parameter which will return records when either the record itself OR its related models have been updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_deleted_at: Nullable[datetime]
    r"""The date and time the object was deleted in the remote system. Objects are automatically marked as deleted when Kombo can't retrieve them from the remote system anymore. Kombo will also anonymize entries 14 days after they disappear.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_created_at: Nullable[datetime]
    r"""The date and time the object was created in the remote system.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    remote_updated_at: Nullable[datetime]
    r"""A timestamp retrieved from the remote system, describing when the resource was last updated.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    current_stage: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseCurrentStage]

    job: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseJob]

    candidate: Nullable[PostAtsJobsJobIDApplicationsPositiveResponseCandidate]

    screening_question_answers: OptionalNullable[
        List[PostAtsJobsJobIDApplicationsPositiveResponseScreeningQuestionAnswerUnion]
    ] = UNSET
    r"""A list of answers to screening questions. The screening answer type `FILE` is currently unsupported."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["screening_question_answers"])
        nullable_fields = set(
            [
                "remote_id",
                "outcome",
                "rejection_reason_name",
                "rejected_at",
                "current_stage_id",
                "job_id",
                "candidate_id",
                "screening_question_answers",
                "custom_fields",
                "remote_url",
                "remote_deleted_at",
                "remote_created_at",
                "remote_updated_at",
                "current_stage",
                "job",
                "candidate",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsJobsJobIDApplicationsPositiveResponseWarningTypedDict(TypedDict):
    message: str


class PostAtsJobsJobIDApplicationsPositiveResponseWarning(BaseModel):
    message: str


class PostAtsJobsJobIDApplicationsPositiveResponseTypedDict(TypedDict):
    data: PostAtsJobsJobIDApplicationsPositiveResponseDataTypedDict
    warnings: List[PostAtsJobsJobIDApplicationsPositiveResponseWarningTypedDict]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""
    status: Literal["success"]


class PostAtsJobsJobIDApplicationsPositiveResponse(BaseModel):
    data: PostAtsJobsJobIDApplicationsPositiveResponseData

    warnings: List[PostAtsJobsJobIDApplicationsPositiveResponseWarning]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"


try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionUnknown.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseAnswer6.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionDate.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionNumber.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionBoolean.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionMultiSelect.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionSingleSelect.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponseQuestionText.model_rebuild()
except NameError:
    pass
try:
    PostAtsJobsJobIDApplicationsPositiveResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel
from kombo.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal
from typing_extensions import Annotated, TypedDict


class PutAtsApplicationsApplicationIDStagePositiveResponseDataTypedDict(TypedDict):
    pass


class PutAtsApplicationsApplicationIDStagePositiveResponseData(BaseModel):
    pass


class PutAtsApplicationsApplicationIDStagePositiveResponseWarningTypedDict(TypedDict):
    message: str


class PutAtsApplicationsApplicationIDStagePositiveResponseWarning(BaseModel):
    message: str


class PutAtsApplicationsApplicationIDStagePositiveResponseTypedDict(TypedDict):
    data: PutAtsApplicationsApplicationIDStagePositiveResponseDataTypedDict
    warnings: List[PutAtsApplicationsApplicationIDStagePositiveResponseWarningTypedDict]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""
    status: Literal["success"]


class PutAtsApplicationsApplicationIDStagePositiveResponse(BaseModel):
    data: PutAtsApplicationsApplicationIDStagePositiveResponseData

    warnings: List[PutAtsApplicationsApplicationIDStagePositiveResponseWarning]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"


try:
    PutAtsApplicationsApplicationIDStagePositiveResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Literal
from typing_extensions import TypedDict


KomboGeneralErrorStatus = Literal["error",]


KomboGeneralErrorCode = Literal[
    "PLATFORM.RATE_LIMIT_EXCEEDED",
    "PLATFORM.CONCURRENCY_LIMIT_EXCEEDED",
    "PLATFORM.INTEGRATION_NOT_FOUND",
    "PLATFORM.INPUT_INVALID",
    "PLATFORM.UNKNOWN_ERROR",
    "PLATFORM.IP_NOT_WHITELISTED",
    "PLATFORM.AUTHENTICATION_INVALID",
    "PLATFORM.TASK_TIMED_OUT",
]
r"""Some errors include an error code that can be used to identify their cause. See the [Error Handling Docs](https://docs.kombo.dev/guides/errors) for more information. For your error handling logic please use the error `code` instead of other properties (e.g. message, http status code, ...)."""


class KomboGeneralErrorErrorTypedDict(TypedDict):
    r"""Error details with structured code for programmatic handling."""

    code: Nullable[KomboGeneralErrorCode]
    r"""Some errors include an error code that can be used to identify their cause. See the [Error Handling Docs](https://docs.kombo.dev/guides/errors) for more information. For your error handling logic please use the error `code` instead of other properties (e.g. message, http status code, ...)."""
    title: Nullable[str]
    r"""A static, human-readable label."""
    message: str
    r"""A dynamic, detailed description of what went wrong in this specific instance."""
    log_url: Nullable[str]
    r"""The log page in the Kombo UI lists every interaction with full details. If you need assistance, share that link with our support team."""


class KomboGeneralErrorError(BaseModel):
    r"""Error details with structured code for programmatic handling."""

    code: Nullable[KomboGeneralErrorCode]
    r"""Some errors include an error code that can be used to identify their cause. See the [Error Handling Docs](https://docs.kombo.dev/guides/errors) for more information. For your error handling logic please use the error `code` instead of other properties (e.g. message, http status code, ...)."""

    title: Nullable[str]
    r"""A static, human-readable label."""

    message: str
    r"""A dynamic, detailed description of what went wrong in this specific instance."""

    log_url: Nullable[str]
    r"""The log page in the Kombo UI lists every interaction with full details. If you need assistance, share that link with our support team."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .gethrisperformancereviewspositiveresponse import (
    GetHrisPerformanceReviewsPositiveResponse,
    GetHrisPerformanceReviewsPositiveResponseTypedDict,
)
from datetime import datetime
from kombo.types import BaseModel, UNSET_SENTINEL
from kombo.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Awaitable, Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class GetHrisPerformanceReviewsGlobalsTypedDict(TypedDict):
    integration_id: NotRequired[str]
    r"""ID of the integration you want to interact with."""


class GetHrisPerformanceReviewsGlobals(BaseModel):
    integration_id: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Integration-Id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the integration you want to interact with."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["integration_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetHrisPerformanceReviewsRequestTypedDict(TypedDict):
    cursor: NotRequired[str]
    r"""An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response."""
    page_size: NotRequired[int]
    r"""The number of results to return per page. Maximum is 250."""
    updated_after: NotRequired[datetime]
    r"""Filter the entries based on the modification date in format `YYYY-MM-DDTHH:mm:ss.sssZ`. Returns records where either the record itself **OR** its nested data has been updated since this timestamp, even if the record's own `changed_at` field remains unchanged.

    If you want to track entry deletion, also set the `include_deleted=true` query parameter, because otherwise, deleted entries will be hidden.

    For more details, see [Understanding changed_at vs updated_after Behavior](https://docs.kombo.dev/ats/getting-started/fetching-data#understanding-changed_at-vs-updated_after-behavior).

    For this endpoint, `updated_after` considers changes to the record itself as well as changes to the following relations:

    - ✓ `review_cycle`
    - ✓ `reviewee`
    - ✓ `reviewer`
    """
    include_deleted: NotRequired[bool]
    r"""By default, deleted entries are not returned. Use the `include_deleted` query param to include deleted entries too."""
    ignore_unsupported_filters: NotRequired[bool]
    r"""When set to `true`, filters targeting fields not supported by this integration will be ignored instead of filtering out all results."""
    ids: NotRequired[List[str]]
    r"""Filter by a comma-separated list of IDs such as `222k7eCGyUdgt2JWZDNnkDs3,B5DVmypWENfU6eMe6gYDyJG3`."""
    remote_ids: NotRequired[List[str]]
    r"""Filter by a comma-separated list of remote IDs."""
    types: NotRequired[List[str]]
    r"""Filter by a comma-separated list of `MANAGER`, `DIRECT_REPORT`, `PEER`, `SELF`

    Leave this blank to get results matching all values.
    """
    review_cycle_ids: NotRequired[List[str]]
    r"""Filter by a comma-separated list of review cycle IDs."""
    reviewee_ids: NotRequired[List[str]]
    r"""Filter by a comma-separated list of reviewee IDs."""


class GetHrisPerformanceReviewsRequest(BaseModel):
    cursor: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response."""

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100
    r"""The number of results to return per page. Maximum is 250."""

    updated_after: Annotated[
        Optional[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter the entries based on the modification date in format `YYYY-MM-DDTHH:mm:ss.sssZ`. Returns records where either the record itself **OR** its nested data has been updated since this timestamp, even if the record's own `changed_at` field remains unchanged.

    If you want to track entry deletion, also set the `include_deleted=true` query parameter, because otherwise, deleted entries will be hidden.

    For more details, see [Understanding changed_at vs updated_after Behavior](https://docs.kombo.dev/ats/getting-started/fetching-data#understanding-changed_at-vs-updated_after-behavior).

    For this endpoint, `updated_after` considers changes to the record itself as well as changes to the following relations:

    - ✓ `review_cycle`
    - ✓ `reviewee`
    - ✓ `reviewer`
    """

    include_deleted: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""By default, deleted entries are not returned. Use the `include_deleted` query param to include deleted entries too."""

    ignore_unsupported_filters: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""When set to `true`, filters targeting fields not supported by this integration will be ignored instead of filtering out all results."""

    ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter by a comma-separated list of IDs such as `222k7eCGyUdgt2JWZDNnkDs3,B5DVmypWENfU6eMe6gYDyJG3`."""

    remote_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter by a comma-separated list of remote IDs."""

    types: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter by a comma-separated list of `MANAGER`, `DIRECT_REPORT`, `PEER`, `SELF`

    Leave this blank to get results matching all values.
    """

    review_cycle_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter by a comma-separated list of review cycle IDs."""

    reviewee_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter by a comma-separated list of reviewee IDs."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cursor",
                "page_size",
                "updated_after",
                "include_deleted",
                "ignore_unsupported_filters",
                "ids",
                "remote_ids",
                "types",
                "review_cycle_ids",
                "reviewee_ids",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetHrisPerformanceReviewsResponseTypedDict(TypedDict):
    result: GetHrisPerformanceReviewsPositiveResponseTypedDict


class GetHrisPerformanceReviewsResponse(BaseModel):
    next: Union[
        Callable[[], Optional[GetHrisPerformanceReviewsResponse]],
        Callable[[], Awaitable[Optional[GetHrisPerformanceReviewsResponse]]],
    ]

    result: GetHrisPerformanceReviewsPositiveResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getintegrationsintegrationidintegrationfieldspositiveresponse import (
    GetIntegrationsIntegrationIDIntegrationFieldsPositiveResponse,
    GetIntegrationsIntegrationIDIntegrationFieldsPositiveResponseTypedDict,
)
from kombo.types import BaseModel, UNSET_SENTINEL
from kombo.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Awaitable, Callable, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class GetIntegrationsIntegrationIDIntegrationFieldsRequestTypedDict(TypedDict):
    integration_id: str
    r"""GET /integrations/:integration_id/integration-fields Parameter"""
    cursor: NotRequired[str]
    r"""An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response."""
    page_size: NotRequired[int]
    r"""The number of results to return per page. Maximum is 2000."""


class GetIntegrationsIntegrationIDIntegrationFieldsRequest(BaseModel):
    integration_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""GET /integrations/:integration_id/integration-fields Parameter"""

    cursor: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response."""

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100
    r"""The number of results to return per page. Maximum is 2000."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["cursor", "page_size"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetIntegrationsIntegrationIDIntegrationFieldsResponseTypedDict(TypedDict):
    result: GetIntegrationsIntegrationIDIntegrationFieldsPositiveResponseTypedDict


class GetIntegrationsIntegrationIDIntegrationFieldsResponse(BaseModel):
    next: Union[
        Callable[[], Optional[GetIntegrationsIntegrationIDIntegrationFieldsResponse]],
        Callable[
            [],
            Awaitable[Optional[GetIntegrationsIntegrationIDIntegrationFieldsResponse]],
        ],
    ]

    result: GetIntegrationsIntegrationIDIntegrationFieldsPositiveResponse



"""Tests for basic SDK behavior."""

from inline_snapshot import snapshot
from tests.conftest import MockContext


class TestBasicSDKBehavior:
    """Test basic SDK behavior."""

    def test_should_include_api_key_in_authorization_header(self):
        """Test that API key is included in Authorization header."""
        ctx = MockContext(api_key="my-custom-api-key")

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {"results": [], "next": None},
                },
            },
        )

        jobs = ctx.kombo.ats.get_jobs()
        if jobs is not None:
            _ = jobs.next()  # Consume first page

        request = ctx.get_last_request()
        assert request.headers.get("authorization") == snapshot("Bearer my-custom-api-key")

    def test_should_include_integration_id_in_x_integration_id_header_when_specified(self):
        """Test that X-Integration-Id header is included when specified."""
        ctx = MockContext(
            api_key="test-key",
            integration_id="my-integration-123",
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {"results": [], "next": None},
                },
            },
        )

        jobs = ctx.kombo.ats.get_jobs()
        if jobs is not None:
            _ = jobs.next()  # Consume first page

        request = ctx.get_last_request()
        assert request.headers.get("x-integration-id") == snapshot("my-integration-123")

    def test_should_not_include_x_integration_id_header_when_not_provided(self):
        """Test that X-Integration-Id header is not included when not provided."""
        ctx = MockContext(
            api_key="test-key",
            integration_id=None,
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {"results": [], "next": None},
                },
            },
        )

        jobs = ctx.kombo.ats.get_jobs()
        if jobs is not None:
            _ = jobs.next()  # Consume first page

        request = ctx.get_last_request()
        # When integration ID is None, the header should not be set
        assert request.headers.get("x-integration-id") is None

    def test_should_correctly_encode_comma_separated_query_parameters(self):
        """Test that comma-separated query parameters are correctly encoded."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [],
                        "next": None,
                    },
                },
            },
        )

        # Make the API call
        jobs = ctx.kombo.ats.get_jobs(
            statuses=["OPEN", "CLOSED"],
            ids=["CPDifhHr7izJhKHmGPkXqknC", "J7znt8TJRiwPVA7paC2iCh8u"],
        )
        if jobs is not None:
            _ = jobs.next()  # Consume first page

        # Verify and snapshot the request details
        request = ctx.get_last_request()
        assert request.path == snapshot(
            '/v1/ats/jobs?page_size=100&include_deleted=false&ignore_unsupported_filters=false&ids=CPDifhHr7izJhKHmGPkXqknC%2CJ7znt8TJRiwPVA7paC2iCh8u&statuses=OPEN%2CCLOSED'
        )

    def test_should_correctly_encode_boolean_query_parameters(self):
        """Test that boolean query parameters are correctly encoded."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {"results": [], "next": None},
                },
            },
        )

        # Test with boolean true
        jobs_with_deleted = ctx.kombo.ats.get_jobs(include_deleted=True)
        if jobs_with_deleted is not None:
            _ = jobs_with_deleted.next()  # Consume first page

        request_with_deleted = ctx.get_last_request()
        assert "include_deleted=true" in request_with_deleted.path

        ctx.clear()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/jobs",
            response={
                "body": {
                    "status": "success",
                    "data": {"results": [], "next": None},
                },
            },
        )

        # Test with boolean false
        jobs_without_deleted = ctx.kombo.ats.get_jobs(include_deleted=False)
        if jobs_without_deleted is not None:
            _ = jobs_without_deleted.next()  # Consume first page

        request_without_deleted = ctx.get_last_request()
        assert "include_deleted=false" in request_without_deleted.path

    def test_should_correctly_serialize_post_request_body(self):
        """Test that POST request bodies are correctly serialized."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="POST",
            path="/v1/ats/jobs/test-job-id/applications",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "id": "app-123",
                        "remote_id": "remote-app-123",
                        "outcome": "PENDING",
                        "rejection_reason_name": None,
                        "rejected_at": None,
                        "current_stage_id": "stage-1",
                        "job_id": "test-job-id",
                        "candidate_id": "candidate-456",
                        "custom_fields": {},
                        "remote_url": "https://example.com/application/123",
                        "changed_at": "2024-01-01T00:00:00Z",
                        "remote_deleted_at": None,
                        "remote_created_at": "2024-01-01T00:00:00Z",
                        "remote_updated_at": "2024-01-01T00:00:00Z",
                        "current_stage": None,
                        "job": None,
                        "candidate": None,
                    },
                    "warnings": [],
                },
            },
        )

        # Make the API call
        ctx.kombo.ats.create_application(
            job_id="test-job-id",
            candidate={
                "first_name": "Jane",
                "last_name": "Smith",
                "email_address": "jane.smith@example.com",
            },
        )

        # Verify request body is correctly serialized
        request = ctx.get_last_request()
        assert request.method == "POST"
        assert request.body == snapshot(
            {
                "candidate": {
                    "first_name": "Jane",
                    "last_name": "Smith",
                    "email_address": "jane.smith@example.com",
                }
            }
        )




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .schema1_union_1 import Schema1Union1, Schema1Union1TypedDict
from .schema2_union_2 import Schema2Union2, Schema2Union2TypedDict
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import Field, model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


GetHrisEmployeesFormPositiveResponseUnifiedKey8 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class GetHrisEmployeesFormPositiveResponseFileRestrictionsTypedDict(TypedDict):
    accepted_mime_types: List[str]
    max_file_size: NotRequired[Nullable[float]]


class GetHrisEmployeesFormPositiveResponseFileRestrictions(BaseModel):
    accepted_mime_types: List[str]

    max_file_size: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["max_file_size"])
        nullable_fields = set(["max_file_size"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PropertiesFileTypedDict(TypedDict):
    label: str
    required: bool
    file_restrictions: GetHrisEmployeesFormPositiveResponseFileRestrictionsTypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey8]]
    type: Literal["file"]


class PropertiesFile(BaseModel):
    label: str

    required: bool

    file_restrictions: GetHrisEmployeesFormPositiveResponseFileRestrictions

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey8] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["file"], AfterValidator(validate_const("file"))],
        pydantic.Field(alias="type"),
    ] = "file"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key"])
        nullable_fields = set(["description", "unified_key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey7 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesArrayTypedDict(TypedDict):
    label: str
    required: bool
    item_type: Schema2Union2TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey7]]
    type: Literal["array"]
    min_items: NotRequired[Nullable[float]]
    max_items: NotRequired[Nullable[float]]


class PropertiesArray(BaseModel):
    label: str

    required: bool

    item_type: Schema2Union2

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey7] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["array"], AfterValidator(validate_const("array"))],
        pydantic.Field(alias="type"),
    ] = "array"

    min_items: OptionalNullable[float] = UNSET

    max_items: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key", "min_items", "max_items"])
        nullable_fields = set(["description", "unified_key", "min_items", "max_items"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey6 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesObjectTypedDict(TypedDict):
    label: str
    required: bool
    properties: Dict[str, Schema1Union1TypedDict]
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey6]]
    type: Literal["object"]


class PropertiesObject(BaseModel):
    label: str

    required: bool

    properties: Dict[str, Schema1Union1]

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey6] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["object"], AfterValidator(validate_const("object"))],
        pydantic.Field(alias="type"),
    ] = "object"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key"])
        nullable_fields = set(["description", "unified_key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey5 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesCheckboxTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey5]]
    type: Literal["checkbox"]


class PropertiesCheckbox(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey5] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["checkbox"], AfterValidator(validate_const("checkbox"))],
        pydantic.Field(alias="type"),
    ] = "checkbox"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key"])
        nullable_fields = set(["description", "unified_key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class GetHrisEmployeesFormPositiveResponseOptionsReferenced2TypedDict(TypedDict):
    link: str
    type: Literal["referenced"]


class GetHrisEmployeesFormPositiveResponseOptionsReferenced2(BaseModel):
    link: str

    TYPE: Annotated[
        Annotated[Literal["referenced"], AfterValidator(validate_const("referenced"))],
        pydantic.Field(alias="type"),
    ] = "referenced"


GetHrisEmployeesFormPositiveResponseRemoteID2TypedDict = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseRemoteID2TypedDict", Union[str, float]
)


GetHrisEmployeesFormPositiveResponseRemoteID2 = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseRemoteID2", Union[str, float]
)


class GetHrisEmployeesFormPositiveResponseEntry2TypedDict(TypedDict):
    id: str
    label: str
    remote_id: GetHrisEmployeesFormPositiveResponseRemoteID2TypedDict
    unified_value: NotRequired[str]


class GetHrisEmployeesFormPositiveResponseEntry2(BaseModel):
    id: str

    label: str

    remote_id: GetHrisEmployeesFormPositiveResponseRemoteID2

    unified_value: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["unified_value"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetHrisEmployeesFormPositiveResponseOptionsInline2TypedDict(TypedDict):
    entries: List[GetHrisEmployeesFormPositiveResponseEntry2TypedDict]
    type: Literal["inline"]


class GetHrisEmployeesFormPositiveResponseOptionsInline2(BaseModel):
    entries: List[GetHrisEmployeesFormPositiveResponseEntry2]

    TYPE: Annotated[
        Annotated[Literal["inline"], AfterValidator(validate_const("inline"))],
        pydantic.Field(alias="type"),
    ] = "inline"


GetHrisEmployeesFormPositiveResponseOptionsUnion2TypedDict = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseOptionsUnion2TypedDict",
    Union[
        GetHrisEmployeesFormPositiveResponseOptionsInline2TypedDict,
        GetHrisEmployeesFormPositiveResponseOptionsReferenced2TypedDict,
    ],
)


GetHrisEmployeesFormPositiveResponseOptionsUnion2 = Annotated[
    Union[
        GetHrisEmployeesFormPositiveResponseOptionsInline2,
        GetHrisEmployeesFormPositiveResponseOptionsReferenced2,
    ],
    Field(discriminator="TYPE"),
]


class PropertiesMultiSelectTypedDict(TypedDict):
    label: str
    required: bool
    options: GetHrisEmployeesFormPositiveResponseOptionsUnion2TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[str]]
    type: Literal["multi_select"]
    min_items: NotRequired[Nullable[float]]
    max_items: NotRequired[Nullable[float]]


class PropertiesMultiSelect(BaseModel):
    label: str

    required: bool

    options: GetHrisEmployeesFormPositiveResponseOptionsUnion2

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[str] = UNSET

    TYPE: Annotated[
        Annotated[
            Literal["multi_select"], AfterValidator(validate_const("multi_select"))
        ],
        pydantic.Field(alias="type"),
    ] = "multi_select"

    min_items: OptionalNullable[float] = UNSET

    max_items: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key", "min_items", "max_items"])
        nullable_fields = set(["description", "unified_key", "min_items", "max_items"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey4 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class GetHrisEmployeesFormPositiveResponseOptionsReferenced1TypedDict(TypedDict):
    link: str
    type: Literal["referenced"]


class GetHrisEmployeesFormPositiveResponseOptionsReferenced1(BaseModel):
    link: str

    TYPE: Annotated[
        Annotated[Literal["referenced"], AfterValidator(validate_const("referenced"))],
        pydantic.Field(alias="type"),
    ] = "referenced"


GetHrisEmployeesFormPositiveResponseRemoteID1TypedDict = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseRemoteID1TypedDict", Union[str, float]
)


GetHrisEmployeesFormPositiveResponseRemoteID1 = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseRemoteID1", Union[str, float]
)


class GetHrisEmployeesFormPositiveResponseEntry1TypedDict(TypedDict):
    id: str
    label: str
    remote_id: GetHrisEmployeesFormPositiveResponseRemoteID1TypedDict
    unified_value: NotRequired[str]


class GetHrisEmployeesFormPositiveResponseEntry1(BaseModel):
    id: str

    label: str

    remote_id: GetHrisEmployeesFormPositiveResponseRemoteID1

    unified_value: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["unified_value"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetHrisEmployeesFormPositiveResponseOptionsInline1TypedDict(TypedDict):
    entries: List[GetHrisEmployeesFormPositiveResponseEntry1TypedDict]
    type: Literal["inline"]


class GetHrisEmployeesFormPositiveResponseOptionsInline1(BaseModel):
    entries: List[GetHrisEmployeesFormPositiveResponseEntry1]

    TYPE: Annotated[
        Annotated[Literal["inline"], AfterValidator(validate_const("inline"))],
        pydantic.Field(alias="type"),
    ] = "inline"


GetHrisEmployeesFormPositiveResponseOptionsUnion1TypedDict = TypeAliasType(
    "GetHrisEmployeesFormPositiveResponseOptionsUnion1TypedDict",
    Union[
        GetHrisEmployeesFormPositiveResponseOptionsInline1TypedDict,
        GetHrisEmployeesFormPositiveResponseOptionsReferenced1TypedDict,
    ],
)


GetHrisEmployeesFormPositiveResponseOptionsUnion1 = Annotated[
    Union[
        GetHrisEmployeesFormPositiveResponseOptionsInline1,
        GetHrisEmployeesFormPositiveResponseOptionsReferenced1,
    ],
    Field(discriminator="TYPE"),
]


class PropertiesSingleSelectTypedDict(TypedDict):
    label: str
    required: bool
    options: GetHrisEmployeesFormPositiveResponseOptionsUnion1TypedDict
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey4]]
    type: Literal["single_select"]


class PropertiesSingleSelect(BaseModel):
    label: str

    required: bool

    options: GetHrisEmployeesFormPositiveResponseOptionsUnion1

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey4] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[
            Literal["single_select"], AfterValidator(validate_const("single_select"))
        ],
        pydantic.Field(alias="type"),
    ] = "single_select"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key"])
        nullable_fields = set(["description", "unified_key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey3 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesDateTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey3]]
    type: Literal["date"]


class PropertiesDate(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey3] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["date"], AfterValidator(validate_const("date"))],
        pydantic.Field(alias="type"),
    ] = "date"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key"])
        nullable_fields = set(["description", "unified_key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey2 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesNumberTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey2]]
    type: Literal["number"]
    min: NotRequired[Nullable[float]]
    max: NotRequired[Nullable[float]]


class PropertiesNumber(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey2] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["number"], AfterValidator(validate_const("number"))],
        pydantic.Field(alias="type"),
    ] = "number"

    min: OptionalNullable[float] = UNSET

    max: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "unified_key", "min", "max"])
        nullable_fields = set(["description", "unified_key", "min", "max"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


GetHrisEmployeesFormPositiveResponseUnifiedKey1 = Literal[
    "first_name",
    "last_name",
    "date_of_birth",
    "gender",
    "home_address.city",
    "home_address.country",
    "home_address.state",
    "home_address.street_1",
    "home_address.street_2",
    "home_address.zip_code",
    "job_title",
    "legal_entity_id",
    "location_id",
    "mobile_phone_number",
    "home_phone_number",
    "nationality",
    "start_date",
    "work_email",
    "private_email",
    "yearly_salary",
]


class PropertiesTextTypedDict(TypedDict):
    label: str
    required: bool
    description: NotRequired[Nullable[str]]
    unified_key: NotRequired[Nullable[GetHrisEmployeesFormPositiveResponseUnifiedKey1]]
    type: Literal["text"]
    min_length: NotRequired[Nullable[float]]
    max_length: NotRequired[Nullable[float]]
    reg_exp: NotRequired[Nullable[str]]


class PropertiesText(BaseModel):
    label: str

    required: bool

    description: OptionalNullable[str] = UNSET

    unified_key: OptionalNullable[GetHrisEmployeesFormPositiveResponseUnifiedKey1] = (
        UNSET
    )

    TYPE: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"

    min_length: OptionalNullable[float] = UNSET

    max_length: OptionalNullable[float] = UNSET

    reg_exp: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["description", "unified_key", "min_length", "max_length", "reg_exp"]
        )
        nullable_fields = set(
            ["description", "unified_key", "min_length", "max_length", "reg_exp"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


PropertiesTypedDict = TypeAliasType(
    "PropertiesTypedDict",
    Union[
        PropertiesDateTypedDict,
        PropertiesCheckboxTypedDict,
        PropertiesSingleSelectTypedDict,
        PropertiesObjectTypedDict,
        PropertiesFileTypedDict,
        PropertiesNumberTypedDict,
        PropertiesTextTypedDict,
        PropertiesMultiSelectTypedDict,
        PropertiesArrayTypedDict,
    ],
)


Properties = Annotated[
    Union[
        PropertiesText,
        PropertiesNumber,
        PropertiesDate,
        PropertiesSingleSelect,
        PropertiesMultiSelect,
        PropertiesCheckbox,
        PropertiesObject,
        PropertiesArray,
        PropertiesFile,
    ],
    Field(discriminator="TYPE"),
]


class GetHrisEmployeesFormPositiveResponseDataTypedDict(TypedDict):
    properties: Dict[str, PropertiesTypedDict]


class GetHrisEmployeesFormPositiveResponseData(BaseModel):
    properties: Dict[str, Properties]


class GetHrisEmployeesFormPositiveResponseWarningTypedDict(TypedDict):
    message: str


class GetHrisEmployeesFormPositiveResponseWarning(BaseModel):
    message: str


class GetHrisEmployeesFormPositiveResponseTypedDict(TypedDict):
    data: GetHrisEmployeesFormPositiveResponseDataTypedDict
    warnings: List[GetHrisEmployeesFormPositiveResponseWarningTypedDict]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""
    status: Literal["success"]


class GetHrisEmployeesFormPositiveResponse(BaseModel):
    data: GetHrisEmployeesFormPositiveResponseData

    warnings: List[GetHrisEmployeesFormPositiveResponseWarning]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"


try:
    PropertiesFile.model_rebuild()
except NameError:
    pass
try:
    PropertiesArray.model_rebuild()
except NameError:
    pass
try:
    PropertiesObject.model_rebuild()
except NameError:
    pass
try:
    PropertiesCheckbox.model_rebuild()
except NameError:
    pass
try:
    GetHrisEmployeesFormPositiveResponseOptionsReferenced2.model_rebuild()
except NameError:
    pass
try:
    GetHrisEmployeesFormPositiveResponseOptionsInline2.model_rebuild()
except NameError:
    pass
try:
    PropertiesMultiSelect.model_rebuild()
except NameError:
    pass
try:
    GetHrisEmployeesFormPositiveResponseOptionsReferenced1.model_rebuild()
except NameError:
    pass
try:
    GetHrisEmployeesFormPositiveResponseOptionsInline1.model_rebuild()
except NameError:
    pass
try:
    PropertiesSingleSelect.model_rebuild()
except NameError:
    pass
try:
    PropertiesDate.model_rebuild()
except NameError:
    pass
try:
    PropertiesNumber.model_rebuild()
except NameError:
    pass
try:
    PropertiesText.model_rebuild()
except NameError:
    pass
try:
    GetHrisEmployeesFormPositiveResponse.model_rebuild()
except NameError:
    pass



"""Tests for pagination behavior."""

from datetime import datetime
from tests.conftest import MockContext


class TestPaginationBehavior:
    """Test pagination behavior."""

    def test_should_iterate_through_multiple_pages(self):
        """Test that SDK can iterate through multiple pages of results."""
        ctx = MockContext()

        # Mock 3 pages of results
        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag1",
                                "remote_id": None,
                                "name": "Tag 1",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                            {
                                "id": "tag2",
                                "remote_id": None,
                                "name": "Tag 2",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": "cursor_page2",
                    },
                },
            },
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag3",
                                "remote_id": None,
                                "name": "Tag 3",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                            {
                                "id": "tag4",
                                "remote_id": None,
                                "name": "Tag 4",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": "cursor_page3",
                    },
                },
            },
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag5",
                                "remote_id": None,
                                "name": "Tag 5",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": None,
                    },
                },
            },
        )

        page = ctx.kombo.ats.get_tags()
        all_results = []

        # Iterate through all pages
        while page is not None:
            all_results.extend(page.result.data.results)
            page = page.next()

        # Verify all 5 tags were collected
        assert len(all_results) == 5
        assert [tag.id for tag in all_results] == ["tag1", "tag2", "tag3", "tag4", "tag5"]

        # Verify 3 HTTP requests were made
        requests = ctx.get_requests()
        assert len(requests) == 3

    def test_should_pass_cursor_parameter_to_subsequent_requests(self):
        """Test that cursor parameter is passed to subsequent paginated requests."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag1",
                                "remote_id": None,
                                "name": "Tag 1",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": "test_cursor_abc123",
                    },
                },
            },
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag2",
                                "remote_id": None,
                                "name": "Tag 2",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": None,
                    },
                },
            },
        )

        page = ctx.kombo.ats.get_tags()
        # Iterate through all pages
        while page is not None:
            page = page.next()

        requests = ctx.get_requests()
        assert len(requests) == 2

        # First request should NOT include cursor
        assert "cursor=" not in requests[0].path

        # Second request SHOULD include cursor
        assert "cursor=test_cursor_abc123" in requests[1].path

    def test_should_stop_pagination_when_next_is_null(self):
        """Test that pagination stops when next cursor is null."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag1",
                                "remote_id": None,
                                "name": "Tag 1",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                            {
                                "id": "tag2",
                                "remote_id": None,
                                "name": "Tag 2",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": None,
                    },
                },
            },
        )

        page = ctx.kombo.ats.get_tags()
        page_count = []

        while page is not None:
            page_count.append(1)
            page = page.next()

        # Verify only 1 page was returned
        assert len(page_count) == 1

        # Verify only 1 HTTP request was made
        requests = ctx.get_requests()
        assert len(requests) == 1

    def test_should_preserve_query_parameters_across_paginated_requests(self):
        """Test that original query parameters are preserved across paginated requests."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag1",
                                "remote_id": None,
                                "name": "Tag 1",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": "cursor_for_page2",
                    },
                },
            },
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag2",
                                "remote_id": None,
                                "name": "Tag 2",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": None,
                    },
                },
            },
        )

        page = ctx.kombo.ats.get_tags(
            updated_after=datetime(2024, 1, 1, 0, 0, 0)
        )

        # Iterate through all pages
        while page is not None:
            page = page.next()

        requests = ctx.get_requests()
        assert len(requests) == 2

        # Both requests should include the original query parameters
        # Check that updated_after parameter is present (URL encoded)
        assert "updated_after=2024-01-01T00%3A00%3A00" in requests[0].path
        assert "cursor=" not in requests[0].path

        assert "updated_after=2024-01-01T00%3A00%3A00" in requests[1].path
        assert "cursor=cursor_for_page2" in requests[1].path

    def test_should_support_manual_pagination_with_next(self):
        """Test that manual pagination works by calling next() method."""
        ctx = MockContext()

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag1",
                                "remote_id": None,
                                "name": "Tag 1",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": "manual_cursor_xyz",
                    },
                },
            },
        )

        ctx.mock_endpoint(
            method="GET",
            path="/v1/ats/tags",
            response={
                "body": {
                    "status": "success",
                    "data": {
                        "results": [
                            {
                                "id": "tag2",
                                "remote_id": None,
                                "name": "Tag 2",
                                "changed_at": "2024-01-01T00:00:00.000Z",
                                "remote_deleted_at": None,
                            },
                        ],
                        "next": None,
                    },
                },
            },
        )

        page1 = ctx.kombo.ats.get_tags()

        # Verify first page was fetched
        assert page1.result.data.results is not None
        assert len(page1.result.data.results) == 1

        # Manually call next()
        page2_result = page1.next()

        # Verify second page was fetched (should not be null if cursor was read correctly)
        # This will fail if cursor extraction bug exists
        assert page2_result is not None
        if page2_result:
            assert len(page2_result.result.data.results) == 1
            assert page2_result.result.data.results[0].id == "tag2"

        # Verify 2 HTTP requests were made
        requests = ctx.get_requests()
        assert len(requests) == 2
        assert "cursor=manual_cursor_xyz" in requests[1].path




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ContentType = Literal["PLAIN_TEXT",]
r"""Content type of the note. Currently only `PLAIN_TEXT` is supported."""


class PostAtsApplicationsApplicationIDNotesRequestBodyTeamtailorTypedDict(TypedDict):
    r"""Teamtailor specific remote fields for the note."""

    user_id: NotRequired[str]
    r"""ID of the user that created the note. Defaults to the first admin user found."""


class PostAtsApplicationsApplicationIDNotesRequestBodyTeamtailor(BaseModel):
    r"""Teamtailor specific remote fields for the note."""

    user_id: Optional[str] = None
    r"""ID of the user that created the note. Defaults to the first admin user found."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["user_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyPostHeadersTypedDict(TypedDict):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: NotRequired[Nullable[str]]
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""


class PostAtsApplicationsApplicationIDNotesRequestBodyPostHeaders(BaseModel):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: Annotated[
        OptionalNullable[str], pydantic.Field(alias="On-Behalf-Of")
    ] = UNSET
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["On-Behalf-Of"])
        nullable_fields = set(["On-Behalf-Of"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyGreenhouseTypedDict(TypedDict):
    r"""Fields specific to Greenhouse."""

    post_headers: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyPostHeadersTypedDict
    ]
    r"""Headers we will pass with `POST` requests to Greenhouse."""


class PostAtsApplicationsApplicationIDNotesRequestBodyGreenhouse(BaseModel):
    r"""Fields specific to Greenhouse."""

    post_headers: Optional[
        PostAtsApplicationsApplicationIDNotesRequestBodyPostHeaders
    ] = None
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["post_headers"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyRecruiteeTypedDict(TypedDict):
    r"""Recruitee specific remote fields for the note."""

    visibility: NotRequired[Any]
    r"""Visibility of the created note."""
    is_json: NotRequired[bool]
    r"""Whether the note is in a stringified JSON format. If true, content should contain a valid JSON as per the [Recruitee API documentation](https://docs.recruitee.com/reference/candidatesidnotes) (body_json field). If false we add the note as a plain text."""


class PostAtsApplicationsApplicationIDNotesRequestBodyRecruitee(BaseModel):
    r"""Recruitee specific remote fields for the note."""

    visibility: Optional[Any] = None
    r"""Visibility of the created note."""

    is_json: Optional[bool] = None
    r"""Whether the note is in a stringified JSON format. If true, content should contain a valid JSON as per the [Recruitee API documentation](https://docs.recruitee.com/reference/candidatesidnotes) (body_json field). If false we add the note as a plain text."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["visibility", "is_json"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyBullhornTypedDict(TypedDict):
    r"""Bullhorn specific remote fields for the note."""

    action: NotRequired[str]
    r"""The action (or type) associated with a Note. You can find all available note actions in a Bullhorn instance under System Settings > commentActionList. The default action is `Note`."""


class PostAtsApplicationsApplicationIDNotesRequestBodyBullhorn(BaseModel):
    r"""Bullhorn specific remote fields for the note."""

    action: Optional[str] = None
    r"""The action (or type) associated with a Note. You can find all available note actions in a Bullhorn instance under System Settings > commentActionList. The default action is `Note`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["action"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyLeverTypedDict(TypedDict):
    r"""Lever specific remote fields for the note."""

    perform_as: NotRequired[str]
    r"""ID of the Lever user that will show up as having created the note. If not provided, defaults to the user associated with the OAuth token."""


class PostAtsApplicationsApplicationIDNotesRequestBodyLever(BaseModel):
    r"""Lever specific remote fields for the note."""

    perform_as: Optional[str] = None
    r"""ID of the Lever user that will show up as having created the note. If not provided, defaults to the user associated with the OAuth token."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["perform_as"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyWorkableTypedDict(TypedDict):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: NotRequired[str]
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""


class PostAtsApplicationsApplicationIDNotesRequestBodyWorkable(BaseModel):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: Optional[str] = None
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["on_behalf_of_user_remote_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyRemoteFieldsTypedDict(TypedDict):
    r"""Tool specific remote fields for the note."""

    teamtailor: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyTeamtailorTypedDict
    ]
    r"""Teamtailor specific remote fields for the note."""
    greenhouse: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyGreenhouseTypedDict
    ]
    r"""Fields specific to Greenhouse."""
    recruitee: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyRecruiteeTypedDict
    ]
    r"""Recruitee specific remote fields for the note."""
    bullhorn: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyBullhornTypedDict
    ]
    r"""Bullhorn specific remote fields for the note."""
    lever: NotRequired[PostAtsApplicationsApplicationIDNotesRequestBodyLeverTypedDict]
    r"""Lever specific remote fields for the note."""
    workable: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyWorkableTypedDict
    ]
    r"""Workable specific remote fields for ATS actions."""


class PostAtsApplicationsApplicationIDNotesRequestBodyRemoteFields(BaseModel):
    r"""Tool specific remote fields for the note."""

    teamtailor: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyTeamtailor] = (
        None
    )
    r"""Teamtailor specific remote fields for the note."""

    greenhouse: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyGreenhouse] = (
        None
    )
    r"""Fields specific to Greenhouse."""

    recruitee: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyRecruitee] = (
        None
    )
    r"""Recruitee specific remote fields for the note."""

    bullhorn: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyBullhorn] = None
    r"""Bullhorn specific remote fields for the note."""

    lever: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyLever] = None
    r"""Lever specific remote fields for the note."""

    workable: Optional[PostAtsApplicationsApplicationIDNotesRequestBodyWorkable] = None
    r"""Workable specific remote fields for ATS actions."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["teamtailor", "greenhouse", "recruitee", "bullhorn", "lever", "workable"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDNotesRequestBodyTypedDict(TypedDict):
    content: str
    r"""UTF-8 content of the note."""
    content_type: ContentType
    r"""Content type of the note. Currently only `PLAIN_TEXT` is supported."""
    remote_fields: NotRequired[
        PostAtsApplicationsApplicationIDNotesRequestBodyRemoteFieldsTypedDict
    ]
    r"""Tool specific remote fields for the note."""


class PostAtsApplicationsApplicationIDNotesRequestBody(BaseModel):
    content: str
    r"""UTF-8 content of the note."""

    content_type: ContentType
    r"""Content type of the note. Currently only `PLAIN_TEXT` is supported."""

    remote_fields: Optional[
        PostAtsApplicationsApplicationIDNotesRequestBodyRemoteFields
    ] = None
    r"""Tool specific remote fields for the note."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["remote_fields"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PostAtsApplicationsApplicationIDNotesRequestBodyPostHeaders.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


PostAtsCandidatesCandidateIDAttachmentsRequestBodyType = Literal[
    "CV",
    "COVER_LETTER",
    "OTHER",
]
r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyAttachmentTypedDict(TypedDict):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""
    type: PostAtsCandidatesCandidateIDAttachmentsRequestBodyType
    r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""
    content_type: NotRequired[str]
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """
    data_url: NotRequired[str]
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """
    data: NotRequired[str]
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyAttachment(BaseModel):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""

    type: PostAtsCandidatesCandidateIDAttachmentsRequestBodyType
    r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""

    content_type: Optional[str] = None
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """

    data_url: Optional[str] = None
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    data: Optional[str] = None
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["content_type", "data_url", "data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyPostHeadersTypedDict(TypedDict):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: NotRequired[Nullable[str]]
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyPostHeaders(BaseModel):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: Annotated[
        OptionalNullable[str], pydantic.Field(alias="On-Behalf-Of")
    ] = UNSET
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["On-Behalf-Of"])
        nullable_fields = set(["On-Behalf-Of"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyGreenhouseTypedDict(TypedDict):
    r"""Fields specific to Greenhouse."""

    post_headers: NotRequired[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyPostHeadersTypedDict
    ]
    r"""Headers we will pass with `POST` requests to Greenhouse."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyGreenhouse(BaseModel):
    r"""Fields specific to Greenhouse."""

    post_headers: Optional[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyPostHeaders
    ] = None
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["post_headers"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyWorkableTypedDict(TypedDict):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: NotRequired[str]
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyWorkable(BaseModel):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: Optional[str] = None
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["on_behalf_of_user_remote_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyRemoteFieldsTypedDict(
    TypedDict
):
    r"""Additional fields that we will pass through to specific ATS systems."""

    greenhouse: NotRequired[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyGreenhouseTypedDict
    ]
    r"""Fields specific to Greenhouse."""
    workable: NotRequired[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyWorkableTypedDict
    ]
    r"""Workable specific remote fields for ATS actions."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyRemoteFields(BaseModel):
    r"""Additional fields that we will pass through to specific ATS systems."""

    greenhouse: Optional[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyGreenhouse
    ] = None
    r"""Fields specific to Greenhouse."""

    workable: Optional[PostAtsCandidatesCandidateIDAttachmentsRequestBodyWorkable] = (
        None
    )
    r"""Workable specific remote fields for ATS actions."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["greenhouse", "workable"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesCandidateIDAttachmentsRequestBodyTypedDict(TypedDict):
    attachment: PostAtsCandidatesCandidateIDAttachmentsRequestBodyAttachmentTypedDict
    remote_fields: NotRequired[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyRemoteFieldsTypedDict
    ]
    r"""Additional fields that we will pass through to specific ATS systems."""


class PostAtsCandidatesCandidateIDAttachmentsRequestBody(BaseModel):
    attachment: PostAtsCandidatesCandidateIDAttachmentsRequestBodyAttachment

    remote_fields: Optional[
        PostAtsCandidatesCandidateIDAttachmentsRequestBodyRemoteFields
    ] = None
    r"""Additional fields that we will pass through to specific ATS systems."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["remote_fields"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PostAtsCandidatesCandidateIDAttachmentsRequestBodyPostHeaders.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, UNSET_SENTINEL
from kombo.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseType = Literal[
    "DEFAULT",
    "CUSTOM",
]
r"""The type of the integration field. There is a type to avoid collisions between DEFAULT fields and CUSTOM fields with the same key."""


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseDataTypedDict(
    TypedDict
):
    id: str
    r"""The unique ID of the field."""
    key: str
    r"""The key of the field in the remote system."""
    model: str
    r"""The model the field is associated with."""
    type: PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseType
    r"""The type of the integration field. There is a type to avoid collisions between DEFAULT fields and CUSTOM fields with the same key."""
    label: Nullable[str]
    r"""The label of the field in the remote system."""
    is_passthrough_enabled: bool
    r"""Whether the field is included in the integrations_fields array of the corresponding model."""
    is_writable: Literal[False]
    r"""This field is related to a deprecated feature and means nothing, but is kept for backwards compatibility."""


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseData(
    BaseModel
):
    id: str
    r"""The unique ID of the field."""

    key: str
    r"""The key of the field in the remote system."""

    model: str
    r"""The model the field is associated with."""

    type: PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseType
    r"""The type of the integration field. There is a type to avoid collisions between DEFAULT fields and CUSTOM fields with the same key."""

    label: Nullable[str]
    r"""The label of the field in the remote system."""

    is_passthrough_enabled: bool
    r"""Whether the field is included in the integrations_fields array of the corresponding model."""

    IS_WRITABLE: Annotated[
        Annotated[Literal[False], AfterValidator(validate_const(False))],
        pydantic.Field(alias="is_writable"),
    ] = False
    r"""This field is related to a deprecated feature and means nothing, but is kept for backwards compatibility."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseTypedDict(
    TypedDict
):
    data: PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseDataTypedDict
    status: Literal["success"]


class PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponse(
    BaseModel
):
    data: PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseData

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"


try:
    PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponseData.model_rebuild()
except NameError:
    pass
try:
    PatchIntegrationsIntegrationIDIntegrationFieldsIntegrationFieldIDPositiveResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel
from kombo.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal
from typing_extensions import Annotated, TypedDict


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponseDataTypedDict(TypedDict):
    pass


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponseData(BaseModel):
    pass


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponseWarningTypedDict(TypedDict):
    message: str


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponseWarning(BaseModel):
    message: str


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponseTypedDict(TypedDict):
    data: PostHrisEmployeesEmployeeIDDocumentsPositiveResponseDataTypedDict
    warnings: List[PostHrisEmployeesEmployeeIDDocumentsPositiveResponseWarningTypedDict]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""
    status: Literal["success"]


class PostHrisEmployeesEmployeeIDDocumentsPositiveResponse(BaseModel):
    data: PostHrisEmployeesEmployeeIDDocumentsPositiveResponseData

    warnings: List[PostHrisEmployeesEmployeeIDDocumentsPositiveResponseWarning]
    r"""These are the interaction warnings that are shown in the dashboard. They are meant to provide debug information to you. We recommend logging them to the console."""

    STATUS: Annotated[
        Annotated[Literal["success"], AfterValidator(validate_const("success"))],
        pydantic.Field(alias="status"),
    ] = "success"


try:
    PostHrisEmployeesEmployeeIDDocumentsPositiveResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class PutIntegrationsIntegrationIDCustomFieldsCustomFieldIDRequestBodyTypedDict(
    TypedDict
):
    integration_field_id: Nullable[str]
    r"""The integration field's unique ID which the custom field should be mapped to. Null to remove a mapping."""


class PutIntegrationsIntegrationIDCustomFieldsCustomFieldIDRequestBody(BaseModel):
    integration_field_id: Nullable[str]
    r"""The integration field's unique ID which the custom field should be mapped to. Null to remove a mapping."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m
